@@ -4,7 +4,7 @@
  * See http://mod-qos.sourceforge.net/ for further
  * details.
  *
- * Copyright (C) 2019 Pascal Buchbinder
+ * Copyright (C) 2023 Pascal Buchbinder
  *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
@@ -22,7 +22,7 @@
  * limitations under the License.
  */
 
-static const char revision[] = "$Id: qs_util.c 2542 2019-02-22 06:37:10Z pbuchbinder $";
+static const char revision[] = "$Id: qs_util.c 2654 2022-05-13 09:12:42Z pbuchbinder $";
 
 #include <stdio.h>
 #include <pthread.h>
@@ -36,6 +36,11 @@ static const char revision[] = "$Id: qs_util.c 2542 2019-02-22 06:37:10Z pbuchbi
 #include <errno.h>
 #include <pwd.h>
 
+#define PCRE2_CODE_UNIT_WIDTH 8
+#include <pcre2.h>
+typedef pcre2_match_data* match_data_pt;
+typedef size_t*           match_vector_pt;
+
 #include "qs_util.h"
 
 /* mutex for counter access */
@@ -326,3 +331,79 @@ void qs_setuid(const char *username, const char *cmd) {
     }
   }
 }
+/* pcre ------------------------------------------------------- */
+int qs_pregfree(void *p) {
+  qs_regfree((qs_regex_t *)p);
+  return 0;
+}
+
+void qs_regfree(qs_regex_t *preg) {
+  if(preg->state == 1) {
+    pcre2_code_free(preg->re_pcre);
+  }
+}
+
+int qs_regcomp(qs_regex_t *preg, const char *pattern, int cflags) {
+  unsigned int capcount;
+  size_t erroffset;
+  int errcode = 0;
+  int options = cflags;
+  preg->state = 0;
+
+  preg->re_pcre = pcre2_compile((const unsigned char *)pattern,
+				PCRE2_ZERO_TERMINATED, options, &errcode,
+				&erroffset, NULL);
+
+  if (preg->re_pcre == NULL) {
+    return 1;
+  }
+
+  pcre2_pattern_info((const pcre2_code *)preg->re_pcre,
+		     PCRE2_INFO_CAPTURECOUNT, &capcount);
+  preg->re_nsub = capcount;
+
+  preg->state = 1;
+  
+  return 0;
+}
+
+int qs_regexec_len(const qs_regex_t *preg, const char *buff,
+		   unsigned int len, unsigned int nmatch,
+		   qs_regmatch_t *pmatch, int eflags) {
+  int rc;
+  int options = 0;
+  match_vector_pt ovector = NULL;
+  unsigned int ncaps = (unsigned int)preg->re_nsub + 1;
+  match_data_pt data = pcre2_match_data_create(ncaps, NULL);
+
+  if (!data) {
+    return -1;
+  }
+
+  options = eflags;
+
+  rc = pcre2_match((const pcre2_code *)preg->re_pcre,
+		   (const unsigned char *)buff, len,
+		   0, options, data, NULL);
+  ovector = pcre2_get_ovector_pointer(data);
+  
+  if (rc >= 0) {
+    unsigned int n = rc, i;
+    if (n == 0 || n > nmatch)
+      rc = n = nmatch; /* All capture slots were filled in */
+    for (i = 0; i < n; i++) {
+      pmatch[i].rm_so = ovector[i * 2];
+      pmatch[i].rm_eo = ovector[i * 2 + 1];
+    }
+    for (; i < nmatch; i++) {
+      pmatch[i].rm_so = pmatch[i].rm_eo = -1;
+    }
+    pcre2_match_data_free(data);
+    return rc;
+  }
+  else {
+    pcre2_match_data_free(data);
+    
+    return -1;
+  }
+}
