@@ -6,7 +6,7 @@
  * See http://mod-qos.sourceforge.net/ for further
  * details.
  *
- * Copyright (C) 2019 Pascal Buchbinder
+ * Copyright (C) 2023 Pascal Buchbinder
  *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
@@ -25,7 +25,7 @@
  *
  */
 
-static const char revision[] = "$Id: qscheck.c 2542 2019-02-22 06:37:10Z pbuchbinder $";
+static const char revision[] = "$Id: qscheck.c 2654 2022-05-13 09:12:42Z pbuchbinder $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -65,7 +65,7 @@ static char *checkedHosts = NULL;
  */
 static void usage(char *cmd) {
   printf("\n");
-  printf("Monitor programm testing the TCP connectivity to servers.\n");
+  printf("Monitor program testing the TCP connectivity to servers.\n");
   printf("\n");
   printf("Usage: %s -c <httpd.conf> [-v]\n", cmd);
   printf("\n");
