@@ -2,12 +2,12 @@
  */
 /**
  * Filter utilities for the quality of service module mod_qos
- * used to create white list rules for request line filters.
+ * used to create allow list rules for request line filters.
  *
  * See http://mod-qos.sourceforge.net/ for further
  * details.
  *
- * Copyright (C) 2019 Pascal Buchbinder
+ * Copyright (C) 2023 Pascal Buchbinder
  *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
@@ -26,7 +26,7 @@
  *
  */
 
-static const char revision[] = "$Id: qsfilter2.c 2542 2019-02-22 06:37:10Z pbuchbinder $";
+static const char revision[] = "$Id: qsfilter2.c 2654 2022-05-13 09:12:42Z pbuchbinder $";
 
 /* system */
 #include <stdio.h>
@@ -37,8 +37,6 @@ static const char revision[] = "$Id: qsfilter2.c 2542 2019-02-22 06:37:10Z pbuch
 #include <unistd.h>
 #include <time.h>
 
-#include <pcre.h>
-
 /* apr */
 #include <apr.h>
 #include <apr_uri.h>
@@ -60,6 +58,9 @@ static const char revision[] = "$Id: qsfilter2.c 2542 2019-02-22 06:37:10Z pbuch
 /* OpenSSL  */
 #include <openssl/safestack.h>
 
+#define PCRE2_CODE_UNIT_WIDTH 8
+#include <pcre2.h>
+
 #include "qs_util.h"
 
 #define MAX_LINE 32768
@@ -89,11 +90,11 @@ typedef enum  {
 #define QS_OVECCOUNT 3
 
 /* request line detection */
-#define QOSC_REQ          "(OPTIONS|GET|HEAD|POST|PUT|DELETE|TRACE|CONNECT|PROPFIND|PROPPATCH|MKCOL|COPY|MOVE|LOCK|UNLOCK|VERSION-CONTROL|REPORT|CHECKOUT|CHECKIN|UNCHECKOUT|MKWORKSPACE|UPDATE|LABEL|MERGE|BASELINE-CONTROL|MKACTIVITY|ORDERPATCH|ACL|PATCH|SEARCH|BCOPY|BDELETE|BMOVE|BPROPFIND|BPROPPATCH|NOTIFY|POLL|SUBSCRIBE|UNSUBSCRIBE|X-MS-ENUMATTS|RPC_IN_DATA|RPC_OUT_DATA) /[\x20-\x21\x23-\xFF]* HTTP/"
+#define QOSC_REQ          "(OPTIONS|GET|HEAD|POST|PUT|DELETE|TRACE|CONNECT|PROPFIND|PROPPATCH|MKCOL|COPY|MOVE|LOCK|UNLOCK|VERSION-CONTROL|REPORT|CHECKOUT|CHECKIN|UNCHECKOUT|MKWORKSPACE|UPDATE|LABEL|MERGE|BASELINE-CONTROL|MKACTIVITY|ORDERPATCH|ACL|PATCH|SEARCH|BCOPY|BDELETE|BMOVE|BPROPFIND|BPROPPATCH|NOTIFY|POLL|SUBSCRIBE|UNSUBSCRIBE|X-MS-ENUMATTS|RPC_IN_DATA|RPC_OUT_DATA) (/[\x20-\x21\x23-\xFF]*) HTTP/"
 
-pcre *pcre_b64;
-pcre *pcre_hx;
-pcre *pcre_simple_path;
+qs_regex_t *pcre_b64;
+qs_regex_t *pcre_hx;
+qs_regex_t *pcre_simple_path;
 
 #define QOS_DEC_MODE_FLAGS_URL        0x00
 #define QOS_DEC_MODE_FLAGS_HTML       0x01
@@ -113,14 +114,13 @@ static int m_query_single_pcre = 0;
 static int m_query_len_pcre = 10;
 static int m_exit_on_error = 0;
 static int m_handler = 0;
-static pcre *m_req_regex = NULL;
+static qs_regex_t *m_req_regex = NULL;
 static int m_log_req_regex = 0;
 static const char *m_pfx = NULL;
 static const char *m_filter = NULL;
 
 typedef struct {
-  pcre *pcre;
-  pcre_extra *extra;
+  qs_regex_t *pcre;
   char *rule;
   char *path;
   char *query_m_string;
@@ -137,32 +137,30 @@ int STACK_qs_cmp(const char * const *_pA, const char * const *_pB) {
 }
 
 /* compiles a pcre (exit on error) */
-static pcre *qos_pcre_compile(char *pattern, int option) {
-  const char *errptr = NULL;
-  int erroffset;
-  pcre *pcre = pcre_compile(pattern, PCRE_DOTALL|option, &errptr, &erroffset, NULL);
-  if(pcre == NULL) {
-    fprintf(stderr, "ERROR, rule <%s> could not compile pcre at position %d,"
-            " reason: %s\n", pattern, erroffset, errptr);
+static qs_regex_t *qos_pcre_compile(apr_pool_t *pool, char *pattern, int option) {
+  qs_regex_t *preg = apr_palloc(pool, sizeof(qs_regex_t));
+  if(qs_regcomp(preg, pattern, PCRE2_DOTALL|option) != 0) {
+    fprintf(stderr, "ERROR, rule <%s> could not compile pcre\n", pattern);
     exit(1);
   }
-  return pcre;
+  apr_pool_pre_cleanup_register(pool, preg, qs_pregfree);
+  return preg;
 }
 
 /* tries to detect base64/hex patterns (mix of upper and lower case characters) */
 static char *qos_detect_b64(char *line, int silent) {
-  int ovector[QS_OVECCOUNT];
-  int rc_c = pcre_exec(pcre_b64, NULL, line, strlen(line), 0, 0, ovector, QS_OVECCOUNT);
+  qs_regmatch_t regm[QS_MAX_REG_MATCH];
+  int rc_c = qs_regexec_len(pcre_b64, line, strlen(line), QS_MAX_REG_MATCH, regm, 0);
   if(rc_c >= 0) {
     if((m_verbose > 1) && !silent) printf("  B64: %.*s\n",
-                                          ovector[1] - ovector[0], &line[ovector[0]]);
-    return &line[ovector[0]];
+                                          regm[0].rm_eo - regm[0].rm_so, &line[regm[0].rm_so]);
+    return &line[regm[0].rm_so];
   }
-  rc_c = pcre_exec(pcre_hx, NULL, line, strlen(line), 0, 0, ovector, QS_OVECCOUNT);
+  rc_c = qs_regexec_len(pcre_hx, line, strlen(line), QS_MAX_REG_MATCH, regm, 0);
   if(rc_c >= 0) {
     if((m_verbose > 1) && !silent) printf("  HX: %.*s\n",
-                                          ovector[1] - ovector[0], &line[ovector[0]]);
-    return &line[ovector[0]];
+                                          regm[0].rm_eo - regm[0].rm_so, &line[regm[0].rm_so]);
+    return &line[regm[0].rm_so];
   }
   return NULL;
 }
@@ -325,14 +323,14 @@ static int qos_fgetline(char *s, int n, FILE *f) {
 }
 
 /* init global pcre */
-static void qos_init_pcre() {
+static void qos_init_pcre(apr_pool_t *pool) {
   char buf[1024];
   sprintf(buf, "%s{%d,}", QS_B64, m_base64);
-  pcre_b64 = qos_pcre_compile(buf, 0);
+  pcre_b64 = qos_pcre_compile(pool, buf, 0);
   sprintf(buf, "%s{%d,}", QS_HX, m_base64);
-  pcre_hx = qos_pcre_compile(buf, 0);
-  pcre_simple_path = qos_pcre_compile("^"QS_SIMPLE_PATH_PCRE"$", 0);
-  m_req_regex = qos_pcre_compile(QOSC_REQ, 0);
+  pcre_hx = qos_pcre_compile(pool, buf, 0);
+  pcre_simple_path = qos_pcre_compile(pool, "^"QS_SIMPLE_PATH_PCRE"$", 0);
+  m_req_regex = qos_pcre_compile(pool, QOSC_REQ, 0);
 }
 
 static void usage(char *cmd, int man) {
@@ -368,12 +366,12 @@ static void usage(char *cmd, int man) {
   qs_man_print(man, " line. The module supports both, negative and positive security\n");
   qs_man_print(man, " model. The QS_Deny* directives are used to specify request line\n");
   qs_man_print(man, " patterns which are not allowed to access the server (negative\n");
-  qs_man_print(man, " security model / blacklist). These rules are used to restrict\n");
+  qs_man_print(man, " security model / deny list). These rules are used to restrict\n");
   qs_man_print(man, " access to certain resources which should not be available to\n");
   qs_man_print(man, " users or to protect the server from malicious patterns. The\n");
-  qs_man_print(man, " QS_Permit* rules implement a positive security model (whitelist).\n");
+  qs_man_print(man, " QS_Permit* rules implement a positive security model (allow list).\n");
   qs_man_print(man, " These directives are used to define allowed request line patterns.\n");
-  qs_man_print(man, " Request which do not match any of thses patterns are not allowed\n");
+  qs_man_print(man, " Request which do not match any of these patterns are not allowed\n");
   qs_man_print(man, " to access the server.\n");
   if(man) printf("\n\n");
   qs_man_print(man, " %s is an audit log analyzer used to generate filter\n", cmd);
@@ -393,7 +391,7 @@ static void usage(char *cmd, int man) {
   qs_man_print(man, "     Input file containing request URIs.\n");
   qs_man_print(man, "     The URIs for this file have to be extracted from the servers\n");
   qs_man_print(man, "     access logs. Each line of the input file contains a request\n");
-  qs_man_print(man, "     URI consiting of a path and and query.\n");
+  qs_man_print(man, "     URI consisting of a path and and query.\n");
   printf("\n");
   printf("     Example:\n");
   qs_man_println(man, "       /aaa/index.do\n");
@@ -422,10 +420,10 @@ static void usage(char *cmd, int man) {
   qs_man_print(man, "     request URI in the input data (-i) in order to make sure not\n");
   qs_man_print(man, "     to be deleted by the rule optimisation algorithm.\n");
   qs_man_print(man, "     QS_Deny* rules from this file are used to filter request lines\n");
-  qs_man_print(man, "     which should not be used for whitelist rule generation.\n");
+  qs_man_print(man, "     which should not be used for allow list rule generation.\n");
   printf("\n");
   printf("     Example:\n");
-  qs_man_println(man, "       # manually defined whitelist rule:\n");
+  qs_man_println(man, "       # manually defined allow list rule:\n");
   qs_man_println(man, "       QS_PermitUri +view deny \"^[/a-zA-Z0-9]+/view\\?(page=[0-9]+)?$\"\n");
   qs_man_println(man, "       # filter unwanted request line patterns:\n");
   qs_man_println(man, "       QS_DenyRequestLine +printable deny \".*[\\x00-\\x19].*\"\n");
@@ -450,7 +448,7 @@ static void usage(char *cmd, int man) {
   if(man) printf("\n.TP\n");
   qs_man_print(man, "  -p\n");
   if(man) printf("\n");
-  qs_man_print(man, "     Repesents query by pcre only (no literal strings).\n");
+  qs_man_print(man, "     Represents query by pcre only (no literal strings).\n");
   if(man) printf("\n.TP\n");
   qs_man_print(man, "  -s\n");
   if(man) printf("\n");
@@ -458,7 +456,7 @@ static void usage(char *cmd, int man) {
   if(man) printf("\n.TP\n");
   qs_man_print(man, "  -m\n");
   if(man) printf("\n");
-  qs_man_print(man, "     Uses one pcre for multipe query values (recommended mode).\n");
+  qs_man_print(man, "     Uses one pcre for multiple query values (recommended mode).\n");
   if(man) printf("\n.TP\n");
   qs_man_print(man, "  -o\n");
   if(man) printf("\n");
@@ -588,7 +586,7 @@ static apr_table_t *qos_get_used(apr_pool_t *pool, apr_table_t *rules, apr_table
       for(i = 0; i < apr_table_elts(rules)->nelts; i++) {
         if(i != j) {
           qs_rule_t *rs = (qs_rule_t *)entry[i].val;
-          if(pcre_exec(rs->pcre, rs->extra, line, strlen(line), 0, 0, NULL, 0) >= 0) {
+          if(qs_regexec_len(rs->pcre, line, strlen(line), 0, NULL, 0) >= 0) {
             match = 1;
             break;
           }
@@ -814,8 +812,7 @@ static void qos_query_optimization(apr_pool_t *pool, apr_table_t *rules) {
                                qos_qqs(pool, query_m_string, query_m_pcre, 0, 0, 0), NULL);
           }
           rule = apr_pstrcat(pool, rule, "$", NULL);
-          rs->pcre = qos_pcre_compile(rule, 0);
-          rs->extra = pcre_study(rs->pcre, 0, &errptr);
+          rs->pcre = qos_pcre_compile(pool, rule, 0);
           rs->path = r->path;
           apr_table_setn(new, rule, (char *)rs);
           if(m_verbose) {
@@ -898,7 +895,7 @@ static int qos_test_for_existing_rule(char *plain, char *line, apr_table_t *rule
   if((line == 0) || (strlen(line) == 0)) return 0;
   for(i = 0; i < apr_table_elts(rules)->nelts; i++) {
     qs_rule_t *rs = (qs_rule_t *)entry[i].val;
-    if(pcre_exec(rs->pcre, rs->extra, line, strlen(line), 0, 0, NULL, 0) >= 0) {
+    if(qs_regexec_len(rs->pcre, line, strlen(line), 0, NULL, 0) >= 0) {
       if(first && (apr_table_get(source_rules, entry[i].key) == NULL)) {
         apr_table_add(source_rules, entry[i].key, "");
         apr_table_add(rules_url, line, "");
@@ -918,7 +915,7 @@ static int qos_test_for_existing_rule(char *plain, char *line, apr_table_t *rule
   entry = (apr_table_entry_t *)apr_table_elts(special_rules)->elts;
   for(i = 0; i < apr_table_elts(special_rules)->nelts; i++) {
     qs_rule_t *rs = (qs_rule_t *)entry[i].val;
-    if(pcre_exec(rs->pcre, rs->extra, line, strlen(line), 0, 0, NULL, 0) >= 0) {
+    if(qs_regexec_len(rs->pcre, line, strlen(line), 0, NULL, 0) >= 0) {
       if(m_verbose) {
         printf("# ADD line %d: %s\n", line_nr, plain);
         printf("# -(S) %s\n", entry[i].key);
@@ -930,15 +927,15 @@ static int qos_test_for_existing_rule(char *plain, char *line, apr_table_t *rule
   return 0;
 }
 
-/* filter lines we don't want to add to the whitelist */
-static int qos_enforce_blacklist(apr_table_t *rules, const char *line) {
+/* filter lines we don't want to add to the allow list */
+static int qos_enforce_denylist(apr_table_t *rules, const char *line) {
   int i;
   apr_table_entry_t *entry = (apr_table_entry_t *)apr_table_elts(rules)->elts;
   if((line == 0) || (strlen(line) == 0)) return 0;
   for(i = 0; i < apr_table_elts(rules)->nelts; i++) {
     qs_rule_t *rs = (qs_rule_t *)entry[i].val;
-    if(pcre_exec(rs->pcre, rs->extra, line, strlen(line), 0, 0, NULL, 0) == 0) {
-      if(m_verbose > 1) printf(" blacklist match, rule %s\n", entry[i].key);
+    if(qs_regexec_len(rs->pcre, line, strlen(line), 0, NULL, 0) == 0) {
+      if(m_verbose > 1) printf(" deny list match, rule %s\n", entry[i].key);
       return 1;
     }
   }
@@ -977,8 +974,7 @@ static void qos_load_rules(apr_pool_t *pool, apr_table_t *ruletable,
             {
               const char *errptr = NULL;
               char *pattern;
-              pcre *pcre_test;
-              pcre_extra *extra;
+              qs_regex_t *pcre_test;
               qs_rule_t *rs;
               if(p[0] == '"') {
                 int fl = strlen(p)-2;
@@ -987,11 +983,9 @@ static void qos_load_rules(apr_pool_t *pool, apr_table_t *ruletable,
                 int fl = strlen(p);
                 pattern = apr_psprintf(pool, "%.*s", fl, p);
               }
-              pcre_test = qos_pcre_compile(pattern, option);
-              extra = pcre_study(pcre_test, 0, &errptr);
+              pcre_test = qos_pcre_compile(pool, pattern, option);
               rs = apr_pcalloc(pool, sizeof(qs_rule_t));
               rs->pcre = pcre_test;
-              rs->extra = extra;
               apr_table_setn(ruletable, pattern, (char *)rs);
             }
           }
@@ -1002,10 +996,10 @@ static void qos_load_rules(apr_pool_t *pool, apr_table_t *ruletable,
   fclose(f);
 }
 
-static void qos_load_blacklist(apr_pool_t *pool, apr_table_t *blacklist, const char *httpdconf) {
-  qos_load_rules(pool, blacklist, httpdconf, "QS_DenyRequestLine", PCRE_CASELESS);
+static void qos_load_denylist(apr_pool_t *pool, apr_table_t *denylist, const char *httpdconf) {
+  qos_load_rules(pool, denylist, httpdconf, "QS_DenyRequestLine", PCRE2_CASELESS);
 }
-static void qos_load_whitelist(apr_pool_t *pool, apr_table_t *rules, const char *httpdconf) {
+static void qos_load_allowlist(apr_pool_t *pool, apr_table_t *rules, const char *httpdconf) {
   qos_load_rules(pool, rules, httpdconf, "QS_PermitUri", 0);
 }
 
@@ -1114,7 +1108,7 @@ static char *qos_query_string_pcre(apr_pool_t *pool, const char *path) {
     return ret;
   } else {
     return ret;
-    /* it woud be nice to use (see -o):
+    /* it would be nice to use (see -o):
      *  ((a=b)?(c=d)?)* 
      * instead of:
      *  (a=b)?(c=d)? and (c=d)?(a=b)?
@@ -1281,8 +1275,7 @@ static void qos_rule_optimization(apr_pool_t *pool, apr_pool_t *lpool,
                 char *matchx = apr_psprintf(lpool, "[%s]{%d}", qos_2pcre(lpool, match), e-s);
                 char *new = apr_psprintf(pool, "%.*s%s%s", s, entry[i].key, matchx, &entry[i].key[e]);
                 qs_rule_t *rsn = apr_pcalloc(pool, sizeof(qs_rule_t));
-                rsn->pcre = qos_pcre_compile(new, 0);
-                rsn->extra = pcre_study(rsn->pcre, 0, &errptr);
+                rsn->pcre = qos_pcre_compile(pool, new, 0);
                 rsn->path = rs->path;
                 rsn->query_m_string = rs->query_m_string;
                 rsn->query_m_pcre = rs->query_m_pcre;
@@ -1341,18 +1334,13 @@ static void qos_auto_detect(char **raw) {
   char *line = *raw;
   int rc_c = -1;
   if(m_req_regex) {
-    int ovector[QS_OVECCOUNT];
+    qs_regmatch_t regm[QS_MAX_REG_MATCH];
     /* no request line, maybe raw Apache access log? */
-    rc_c = pcre_exec(m_req_regex, NULL, line, strlen(line), 0, 0, ovector, QS_OVECCOUNT);
+    rc_c = qs_regexec_len(m_req_regex, line, strlen(line), QS_MAX_REG_MATCH, regm, 0);
     if(rc_c >= 0) {
-      char *sr;
-      line = &line[ovector[0]];
-      line[ovector[1] - ovector[0]] = '\0';
-      sr = strchr(line, ' ');
-      while(sr[0] == ' ')sr++;
+      char *sr = &line[regm[2].rm_so];
+      sr[regm[2].rm_eo - regm[2].rm_so] = '\0';
       *raw = sr;
-      sr = strrchr(line, ' ');
-      sr[0] = '\0';
     }
   }
   if(rc_c < 0) {
@@ -1374,7 +1362,7 @@ static void qos_auto_detect(char **raw) {
 }
 
 /* process the input file line by line */
-static void qos_process_log(apr_pool_t *pool, apr_table_t *blacklist, apr_table_t *rules,
+static void qos_process_log(apr_pool_t *pool, apr_table_t *denylist, apr_table_t *rules,
                             apr_table_t *rules_url, apr_table_t *special_rules,
                             FILE *f, int *ln, int *dc, int first) {
   char *readline = apr_pcalloc(pool, MAX_BODY_BUFFER);
@@ -1403,7 +1391,7 @@ static void qos_process_log(apr_pool_t *pool, apr_table_t *blacklist, apr_table_
       qos_auto_detect(&line);
     }
     if(apr_uri_parse(lpool, line, &parsed_uri) != APR_SUCCESS) {
-      fprintf(stderr, "ERROR, could parse uri %s\n", line);
+      fprintf(stderr, "ERROR, could not parse uri %s\n", line);
       if(m_exit_on_error) exit(1);
     }
     if(parsed_uri.path == NULL || (parsed_uri.path[0] != '/')) {
@@ -1418,8 +1406,8 @@ static void qos_process_log(apr_pool_t *pool, apr_table_t *blacklist, apr_table_
       char *fragment = NULL;
       char *copy = apr_pstrdup(lpool, line);
       qos_unescaping(copy);
-      if(qos_enforce_blacklist(blacklist, copy)) {
-        fprintf(stderr, "WARNING: blacklist filter match at line %d for %s\n",
+      if(qos_enforce_denylist(denylist, copy)) {
+        fprintf(stderr, "WARNING: deny list filter match at line %d for %s\n",
                 line_nr, line);
         deny_count++;
       } else {
@@ -1454,8 +1442,9 @@ static void qos_process_log(apr_pool_t *pool, apr_table_t *blacklist, apr_table_
               if(m_handler) {
                 path = qos_path_pcre_string(lpool, parsed_uri.path);
               } else {
-                if(pcre_exec(pcre_simple_path, NULL, parsed_uri.path,
-                             strlen(parsed_uri.path), 0, 0, NULL, 0) >= 0) {
+                if(qs_regexec_len(pcre_simple_path,
+                                  parsed_uri.path, strlen(parsed_uri.path),
+                                  0, NULL, 0) >= 0) {
                   path = apr_pstrdup(lpool, QS_SIMPLE_PATH_PCRE);
                 } else {
                   path = qos_path_pcre(lpool, parsed_uri.path);
@@ -1503,8 +1492,7 @@ static void qos_process_log(apr_pool_t *pool, apr_table_t *blacklist, apr_table_
               rs->fragment = 0;
             }
             rule = apr_pstrcat(pool, rule, "$", NULL);
-            rs->pcre = qos_pcre_compile(rule, 0);
-            rs->extra = pcre_study(rs->pcre, 0, &errptr);
+            rs->pcre = qos_pcre_compile(pool, rule, 0);
             rs->path = apr_pstrdup(pool, path);
             if(m_query_multi_pcre && !fragment) {
               rs->query_m_string = apr_pstrdup(pool, query_m_string);
@@ -1519,7 +1507,7 @@ static void qos_process_log(apr_pool_t *pool, apr_table_t *blacklist, apr_table_
               printf("# %.3d %s\n", apr_table_elts(rules)->nelts+1, rule);
               fflush(stdout);
             }
-            if(pcre_exec(rs->pcre, rs->extra, copy, strlen(copy), 0, 0, NULL, 0) < 0) {
+            if(qs_regexec_len(rs->pcre, copy, strlen(copy), 0, NULL, 0) < 0) {
               fprintf(stderr, "ERROR, rule check failed (did not match)!\n");
               fprintf(stderr, " line %d: %s\n", line_nr, line);
               fprintf(stderr, " string: %s\n", copy);
@@ -1562,7 +1550,7 @@ static void qos_process_log(apr_pool_t *pool, apr_table_t *blacklist, apr_table_
   *ln = line_nr;
 }
 
-static void qos_measurement(apr_pool_t *pool, apr_table_t *blacklist, apr_table_t *rules, FILE *f, int *ln) {
+static void qos_measurement(apr_pool_t *pool, apr_table_t *denylist, apr_table_t *rules, FILE *f, int *ln) {
   char *readline = apr_pcalloc(pool, MAX_BODY_BUFFER);
   int line_nr = 0;
   while(!qos_fgetline(readline, MAX_BODY_BUFFER, f)) {
@@ -1590,7 +1578,7 @@ static void qos_measurement(apr_pool_t *pool, apr_table_t *blacklist, apr_table_
       qos_unescaping(copy);
       for(i = 0; i < apr_table_elts(rules)->nelts; i++) {
         qs_rule_t *rs = (qs_rule_t *)entry[i].val;
-        pcre_exec(rs->pcre, NULL, copy, strlen(copy), 0, 0, NULL, 0);
+        qs_regexec_len(rs->pcre, copy, strlen(copy), 0, NULL, 0);
       }
     }
     apr_pool_destroy(lpool);
@@ -1612,17 +1600,17 @@ int main(int argc, const char * const argv[]) {
   apr_pool_t *pool;
   apr_table_t *rules;
   apr_table_t *special_rules;
-  apr_table_t *blacklist;
+  apr_table_t *denylist;
   apr_table_t *rules_url;
-  int blacklist_size = 0;
-  int whitelist_size = 0;
+  int denylist_size = 0;
+  int allowlist_size = 0;
   char *cmd = strrchr(argv[0], '/');
   const char *httpdconf = NULL;
   apr_app_initialize(&argc, &argv, NULL);
   apr_pool_create(&pool, NULL);
   rules = apr_table_make(pool, 10);
   special_rules = apr_table_make(pool, 10);
-  blacklist = apr_table_make(pool, 10);
+  denylist = apr_table_make(pool, 10);
   rules_url = apr_table_make(pool, 10);
   rc = nice(10);
   if(rc == -1) {
@@ -1710,7 +1698,7 @@ int main(int argc, const char * const argv[]) {
     argc--;
     argv++;
   }
-  qos_init_pcre();
+  qos_init_pcre(pool);
 
   if((m_query_pcre && m_query_multi_pcre) ||
      (m_query_pcre && m_query_single_pcre) ||
@@ -1723,10 +1711,10 @@ int main(int argc, const char * const argv[]) {
   }
 
   if(httpdconf) {
-    qos_load_blacklist(pool, blacklist, httpdconf);
-    blacklist_size = apr_table_elts(blacklist)->nelts;
-    qos_load_whitelist(pool, rules, httpdconf);
-    whitelist_size = apr_table_elts(rules)->nelts;
+    qos_load_denylist(pool, denylist, httpdconf);
+    denylist_size = apr_table_elts(denylist)->nelts;
+    qos_load_allowlist(pool, rules, httpdconf);
+    allowlist_size = apr_table_elts(rules)->nelts;
   }
 
   if(access_log == NULL) usage(cmd, 0);
@@ -1735,7 +1723,7 @@ int main(int argc, const char * const argv[]) {
     fprintf(stderr, "ERROR, could not open input file %s\n", access_log);
     exit(1);
   }
-  qos_process_log(pool, blacklist, rules, rules_url, special_rules, f, &line_nr, &deny_count, 1);
+  qos_process_log(pool, denylist, rules, rules_url, special_rules, f, &line_nr, &deny_count, 1);
   fclose(f);
 
   if(m_redundant) {
@@ -1749,10 +1737,10 @@ int main(int argc, const char * const argv[]) {
       fflush(stdout);
     }
     //    if(httpdconf) {
-    //      qos_load_whitelist(pool, rules, httpdconf);
+    //      qos_load_allowlist(pool, rules, httpdconf);
     //    }
     f = fopen(access_log, "r");
-    qos_process_log(pool, blacklist, rules, rules_url, special_rules, f, &xl, &y, 0);
+    qos_process_log(pool, denylist, rules, rules_url, special_rules, f, &xl, &y, 0);
     fclose(f);
   }
 
@@ -1761,7 +1749,7 @@ int main(int argc, const char * const argv[]) {
     apr_time_t tv;
     f = fopen(access_log, "r");
     tv = apr_time_now();
-    qos_measurement(pool, blacklist, rules, f, &lx);
+    qos_measurement(pool, denylist, rules, f, &lx);
     tv = apr_time_now() - tv;
     performance = apr_time_msec(tv) + (apr_time_sec(tv) * 1000);
     performance = performance / lx;
@@ -1804,9 +1792,9 @@ int main(int argc, const char * const argv[]) {
   printf("#  exit on error (-e): %s\n", m_exit_on_error == 1 ? "yes" : "no");
   printf("#  rule file (-c): %s\n", httpdconf == NULL ? "-" : httpdconf);
   if(httpdconf) {
-    printf("#    whitelist (loaded existing rules): %d\n", whitelist_size);
-    printf("#    blacklist (loaded deny rules): %d\n", blacklist_size);
-    printf("#    blacklist matches: %d\n", deny_count);
+    printf("#    allow list (loaded existing rules): %d\n", allowlist_size);
+    printf("#    deny list (loaded deny rules): %d\n", denylist_size);
+    printf("#    deny list matches: %d\n", deny_count);
   }
   printf("#  duration: %ld minutes\n", (end - start) / 60);
   printf("# --------------------------------------------------------\n");
