@@ -9,7 +9,7 @@
  * See http://mod-qos.sourceforge.net/ for further
  * details.
  *
- * Copyright (C) 2019 Pascal Buchbinder
+ * Copyright (C) 2023 Pascal Buchbinder
  *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
@@ -28,7 +28,7 @@
  *
  */
 
-static const char revision[] = "$Id: qslog.c 2542 2019-02-22 06:37:10Z pbuchbinder $";
+static const char revision[] = "$Id: qslog.c 2654 2022-05-13 09:12:42Z pbuchbinder $";
 
 #include <stdio.h>
 #include <string.h>
@@ -45,14 +45,15 @@ static const char revision[] = "$Id: qslog.c 2542 2019-02-22 06:37:10Z pbuchbind
 #include <regex.h>
 #include <time.h>
 
-#include <pcre.h>
-
 /* apr */
 #include <apr.h>
 #include <apr_portable.h>
 #include <apr_support.h>
 #include <apr_strings.h>
 
+#define PCRE2_CODE_UNIT_WIDTH 8
+#include <pcre2.h>
+
 #include "qs_util.h"
 
 /* ----------------------------------
@@ -68,7 +69,7 @@ static const char revision[] = "$Id: qslog.c 2542 2019-02-22 06:37:10Z pbuchbind
 #define NUM_EVENT_TABLES   8
 #define QS_GENERATIONS 14
 #define EVENT_DELIM ','
-#define QSEVENTPATH "QSEVENTPATH" /* varibale name to find event definitions */
+#define QSEVENTPATH "QSEVENTPATH" /* variable name to find event definitions */
 #define QSCOUNTERPATH "QSCOUNTERPATH" /* counter rule definitions */
 #define COUNTER_PATTERN "([a-zA-Z0-9_]+):([a-zA-Z0-9_]+)[-]([0-9]+)[*]([a-zA-Z0-9_]+)[/]([0-9]+)=([0-9]+)"
 
@@ -312,10 +313,10 @@ static int qs_tableSelector(const char *str) {
 }
 
 /**
- * Inserts an event entry and delets expired.
+ * Inserts an event entry and deletes expired.
  *
  * @param l_qs_event Pointer to the event list.
- * @param id Identifer, e.g. IP address or user tracking cookie
+ * @param id Identifier, e.g. IP address or user tracking cookie
  * @param type which counter is used (either 'I' or 'U')
  * @return event counter (number of updates) for the provided id
  */
@@ -439,7 +440,7 @@ static void *gcThread(void *argv) {
 
 /**
  * Helper to print an error message when terminating
- * the programm due to an unexpected error.
+ * the program due to an unexpected error.
  */
 static void qerror(const char *fmt,...) {
   char buf[MAX_LINE];
@@ -977,7 +978,7 @@ static void qs_updateEvents(apr_pool_t *pool, char *E, apr_table_t *events) {
     }
     E = sep;
     if(restore) {
-      // suports multiple parsing of the event string
+      // supports multiple parsing of the event string
       restore[0] = EVENT_DELIM;
     }
   }
@@ -990,31 +991,28 @@ static void qs_updateEvents(apr_pool_t *pool, char *E, apr_table_t *events) {
 static void qsInitCounter(apr_pool_t *pool, apr_table_t *counters) {
   const char *envFile = getenv(QSCOUNTERPATH);
   if(envFile != NULL) {
-    const char *errptr = NULL;
-    int erroffset;
-    int ovector[100];
-    pcre *pcrestat = pcre_compile(COUNTER_PATTERN, PCRE_CASELESS, &errptr, &erroffset, NULL);
+    qs_regmatch_t regm[QS_MAX_REG_MATCH];
+    qs_regex_t *pcrestat = apr_palloc(pool, sizeof(qs_regex_t));
     FILE *file = fopen(envFile, "r"); 
+    if(qs_regcomp(pcrestat, COUNTER_PATTERN, PCRE2_CASELESS) != 0) {
+      fprintf(stderr, "ERROR, could not compile '%s'\n", COUNTER_PATTERN);
+      exit(1);
+    }
+    apr_pool_pre_cleanup_register(pool, pcrestat, qs_pregfree);
     if(file != NULL) {
       char line[MAX_LINE];
       while(!qs_getLinef(line, sizeof(line), file)) {
-        if(pcre_exec(pcrestat, NULL, line, strlen(line), 0, 0, ovector, 100) >= 0) {
+        if(qs_regexec_len(pcrestat, line, strlen(line), QS_MAX_REG_MATCH, regm, 0) >= 0) {
           counter_rec_t *c = apr_pcalloc(pool, sizeof(counter_rec_t));
-          line[ovector[2] + ovector[3] - ovector[2]] = '\0';
-          line[ovector[4] + ovector[5] - ovector[4]] = '\0';
-          line[ovector[6] + ovector[7] - ovector[6]] = '\0';
-          line[ovector[8] + ovector[9] - ovector[8]] = '\0';
-          line[ovector[10] + ovector[11] - ovector[10]] = '\0';
- 
-          c->name = apr_pstrdup(pool, &line[ovector[2]]);
+          c->name = apr_pstrndup(pool, &line[regm[1].rm_so], regm[1].rm_eo - regm[1].rm_so);
           c->count = 0;
           c->total = 0;
           c->start = 0;
-          c->inc = apr_pstrdup(pool, &line[ovector[4]]);
-          c->decVal = atoi(&line[ovector[6]]);
-          c->dec = apr_pstrdup(pool, &line[ovector[8]]);
-          c->duration = atoi(&line[ovector[10]]);
-          c->limit = atoi(&line[ovector[12]]);
+          c->inc = apr_pstrndup(pool, &line[regm[2].rm_so], regm[2].rm_eo - regm[2].rm_so);
+          c->decVal = atoi(apr_pstrndup(pool, &line[regm[3].rm_so], regm[3].rm_eo - regm[3].rm_so));
+          c->dec = apr_pstrndup(pool, &line[regm[4].rm_so], regm[4].rm_eo - regm[4].rm_so);
+          c->duration = atoi(apr_pstrndup(pool, &line[regm[5].rm_so], regm[5].rm_eo - regm[5].rm_so));
+          c->limit = atoi(apr_pstrndup(pool, &line[regm[6].rm_so], regm[6].rm_eo - regm[6].rm_so));
           if(m_verbose) {
             fprintf(stderr, "%s : %s - (%d * %s) / %d = %d\n",
                     c->name, c->inc, c->decVal, c->dec, c->duration, c->limit);
@@ -1599,7 +1597,7 @@ static void updateStat(apr_pool_t *pool, const char *cstr, char *line) {
   char *k = NULL; /* connections (keep alive requests = 0) */
   char *C = NULL; /* custom patter matching the config file */
   char *s = NULL; /* sum */
-  char *a = NULL; /* avarage 1 */
+  char *a = NULL; /* average 1 */
   char *A = NULL; /* average 2 */
   char *M = NULL; /* max */
   char *E = NULL; /* events */
@@ -2086,7 +2084,7 @@ static void usage(const char *cmd, int man) {
   qs_man_println(man, "  - number of requests within measured time (req)\n");
   qs_man_println(man, "  - bytes sent to the client per second ("NBS")\n");
   qs_man_println(man, "  - bytes received from the client per second ("NBIS")\n");
-  qs_man_println(man, "  - repsonse status codes within the last minute (1xx,2xx,3xx,4xx,5xx)\n");
+  qs_man_println(man, "  - response status codes within the last minute (1xx,2xx,3xx,4xx,5xx)\n");
   qs_man_println(man, "  - average response duration ("NAV")\n");
   qs_man_println(man, "  - average response duration in milliseconds ("NAVMS")\n");
   qs_man_println(man, "  - distribution of response durations in seconds within the last minute\n");
@@ -2118,7 +2116,7 @@ static void usage(const char *cmd, int man) {
   qs_man_print(man, "     Defines the log data format and the positions of data\n");
   qs_man_print(man, "     elements processed by this utility.\n");
   qs_man_print(man, "     See to the 'LogFormat' directive of the httpd.conf file\n");
-  qs_man_print(man, "     to see the format defintions of the servers access log data.\n");
+  qs_man_print(man, "     to see the format definitions of the servers access log data.\n");
   if(man) printf("\n");
   qs_man_println(man, "     %s knows the following elements:\n", cmd);
   qs_man_println(man, "     I defines the client ip address (%%h)\n");
@@ -2179,7 +2177,7 @@ static void usage(const char *cmd, int man) {
   if(man) printf("\n.TP\n");
   qs_man_print(man, "  -c <path>\n");
   if(man) printf("\n");
-  qs_man_print(man, "     Enables the collection of log statitics for different request types.\n");
+  qs_man_print(man, "     Enables the collection of log statistics for different request types.\n");
   qs_man_print(man, "     'path' specifies the necessary rule file. Each rule consists of a rule\n");
   qs_man_print(man, "     identifier and a regular expression to identify a request seprarated\n");
   qs_man_print(man, "     by a colon, e.g., 01:^(/a)|(/c). The regular expressions are matched against\n");
