@@ -6,7 +6,7 @@
  * See http://mod-qos.sourceforge.net/ for further
  * details.
  *
- * Copyright (C) 2019 Pascal Buchbinder
+ * Copyright (C) 2023 Pascal Buchbinder
  *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
@@ -25,7 +25,7 @@
  *
  */
 
-static const char revision[] = "$Id: qspng.c 2542 2019-02-22 06:37:10Z pbuchbinder $";
+static const char revision[] = "$Id: qspng.c 2654 2022-05-13 09:12:42Z pbuchbinder $";
 
 #include <stdio.h>
 #include <string.h>
@@ -76,17 +76,17 @@ static const qs_png_elt_t qs_png_elts[] = {
   { "4s", "requests with 4 seconds response time", 25, 95, 180 },
   { "5s", "requests with 5 seconds response time", 15, 90, 180 },
   { ">5s","requests slower than 5 seconds", 35, 90, 185 },
-  { "1xx","requets with HTTP status 1xx",   50, 70, 150 },
-  { "2xx","requets with HTTP status 2xx",   50, 70, 150 },
-  { "3xx","requets with HTTP status 3xx",   50, 70, 150 },
-  { "4xx","requets with HTTP status 4xx",   50, 70, 150 },
-  { "5xx","requets with HTTP status 5xx",   50, 70, 150 },
+  { "1xx","requests with HTTP status 1xx",   50, 70, 150 },
+  { "2xx","requests with HTTP status 2xx",   50, 70, 150 },
+  { "3xx","requests with HTTP status 3xx",   50, 70, 150 },
+  { "4xx","requests with HTTP status 4xx",   50, 70, 150 },
+  { "5xx","requests with HTTP status 5xx",   50, 70, 150 },
   { "ip", "IP addresses",          55, 60, 150 },
   { "usr","active users",          55, 66, 150 },
   { "qV", "created VIP sessions",  55, 50, 155 },
   { "qS", "session pass",          55, 75, 160 },
   { "qD", "access denied",         55, 70, 170 },
-  { "qK", "conection closed",      55, 60, 145 },
+  { "qK", "connection closed",      55, 60, 145 },
   { "qT", "dynamic keep-alive",    55, 55, 153 },
   { "qL", "slow down",             55, 65, 140 },
   { "qA", "connection aborts",     55, 50, 175 },
@@ -400,7 +400,7 @@ static void lp_init(int width, int height, int border, png_bytep **start) {
  * "Main" png function:
  * - reads the data from the file
  * - draws the curve
- * - lables the x axis
+ * - labels the x axis
  *
  * @param width IN size (x axis) of the graph
  * @param height IN size (y axis) of the graph
@@ -426,14 +426,14 @@ static long qs_png_draw(int width, int height, int border,
   char line[HUGE_STRING_LEN];
 
   long peak = 0;           // max of all values
-  double scale = 1;        // scaling factor (heigth x scale = unit)
+  double scale = 1;        // scaling factor (height x scale = unit)
 
   int hour = -1;           // detect "new" hour
-  char date_str[32] = "";  // sting storing the first day (if fist value is at 00h)
+  char date_str[32] = "";  // string storing the first day (if fist value is at 00h)
 
   long ret;
   for(x=0; x<width; x++) hours[x] = 0;
-  /* reads the file and resample measure points to witdh of the graph */
+  /* reads the file and resample measure points to width of the graph */
   while(!qs_png_getline(line, sizeof(line), stat_log) && i < width) {
     char *p = strstr(line, name);
     req[i] = 0;
@@ -486,7 +486,7 @@ static long qs_png_draw(int width, int height, int border,
       sample++;
     }
   }
-  /* calculate y axis scaling (1:1 are heigth pixels) */
+  /* calculate y axis scaling (1:1 are height pixels) */
   if(peak < 10) {
     scale = 0.1;
   } else {
