@@ -4,7 +4,7 @@
  * See http://mod-qos.sourceforge.net/ for further
  * details.
  *
- * Copyright (C) 2019 Pascal Buchbinder
+ * Copyright (C) 2023 Pascal Buchbinder
  *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
@@ -23,7 +23,7 @@
  *
  */
 
-static const char revision[] = "$Id: qsre.c 2542 2019-02-22 06:37:10Z pbuchbinder $";
+static const char revision[] = "$Id: qsre.c 2654 2022-05-13 09:12:42Z pbuchbinder $";
 
 /* system */
 #include <stdio.h>
@@ -34,7 +34,6 @@ static const char revision[] = "$Id: qsre.c 2542 2019-02-22 06:37:10Z pbuchbinde
 #include <time.h>
 
 /* apr */
-#include <pcre.h>
 #include <apr.h>
 #include <apr_strings.h>
 #include <apr_time.h>
@@ -43,6 +42,9 @@ static const char revision[] = "$Id: qsre.c 2542 2019-02-22 06:37:10Z pbuchbinde
 #include <apr_portable.h>
 #include <apr_support.h>
 
+#define PCRE2_CODE_UNIT_WIDTH 8
+#include <pcre2.h>
+
 #include "qs_util.h"
 
 #define QS_OVECCOUNT 100
@@ -87,7 +89,7 @@ static void usage(const char *cmd, int man) {
   if(man) printf(".TP\n");
   qs_man_print(man, "  <string>|<path>\n");
   if(man) printf("\n");
-  qs_man_print(man, "     The first argument either defines a sinlge test string of a path to\n");
+  qs_man_print(man, "     The first argument either defines a single test string of a path to\n");
   qs_man_print(man, "     a file containing either multiple test strings or a test pattern with\n");
   qs_man_print(man, "     newline characters (text).\n");
   if(man) printf("\n.TP\n");
@@ -112,22 +114,19 @@ static void usage(const char *cmd, int man) {
   }
 }
 
-static int rmatch(const char *line, pcre *pcre) {
-  int ovector[QS_OVECCOUNT];
+static int rmatch(const char *line, qs_regex_t *preg) {
+  qs_regmatch_t regm[QS_MAX_REG_MATCH];
   int rc_c = -1;
   do {
-    int rc = pcre_exec(pcre, NULL, line, strlen(line), 0, 0, ovector, QS_OVECCOUNT);
+    int rc = qs_regexec_len(preg, line, strlen(line), QS_MAX_REG_MATCH, regm, 0);
     if(rc >= 0) {
       int ix;
       rc_c = 0;
-      printf("[%.*s]", ovector[1] - ovector[0], &line[ovector[0]]);
+      printf("[%.*s]", regm[0].rm_eo - regm[0].rm_so, &line[regm[0].rm_so]);
       for(ix = 1; ix < rc; ix++) {
-	printf(" $%d=%.*s", ix, ovector[ix*2+1] - ovector[ix*2], &line[ovector[ix*2]]);
-      }
-      line = &line[ovector[1]];
-      if(ovector[1] - ovector[0] == 0) {
-	line++;
+	printf(" $%d=%.*s", ix, regm[ix].rm_eo - regm[ix].rm_so, &line[regm[ix].rm_so]);
       }
+      line = &line[regm[0].rm_eo];
     } else {
       line = NULL;
     }
@@ -138,7 +137,7 @@ static int rmatch(const char *line, pcre *pcre) {
 int main(int argc, const char *const argv[]) {
   const char *errptr = NULL;
   int erroffset;
-  pcre *pcre;
+  qs_regex_t *preg;
   int rc_c = -1;
   const char *line;
   const char *in;
@@ -152,7 +151,6 @@ int main(int argc, const char *const argv[]) {
 
   apr_app_initialize(&argc, &argv, NULL);
   apr_pool_create(&pool, NULL);
-
   if(cmd == NULL) {
     cmd = (char *)argv[0];
   } else {
@@ -186,12 +184,12 @@ int main(int argc, const char *const argv[]) {
   }
   printf("expression: %s\n", pattern);
 
-  pcre = pcre_compile(pattern, PCRE_DOTALL|PCRE_CASELESS, &errptr, &erroffset, NULL);
-  if(pcre == NULL) {
-    fprintf(stderr, "ERROR, rule <%s> could not compile pcre at position %d,"
-	    " reason: %s\n", pattern, erroffset, errptr);
+  preg = apr_palloc(pool, sizeof(qs_regex_t));
+  if(qs_regcomp(preg, pattern, PCRE2_DOTALL|PCRE2_CASELESS) != 0) {
+    fprintf(stderr, "ERROR, rule <%s> could not compile regular expression\n", pattern);
     exit(1);
   }
+  apr_pool_pre_cleanup_register(pool, preg, qs_pregfree);
 
   file = fopen(in, "r");
   if(file) {
@@ -207,17 +205,17 @@ int main(int argc, const char *const argv[]) {
       }
       if(readline[0] >= 32 && strlen(readline) > 0) {
 	line = readline;
-	rc_c = rmatch(line, pcre);
+	rc_c = rmatch(line, preg);
       }
       printf("\n");
     }
     fclose(file);
     printf("entire content match:\n");
-    rc_c = rmatch(raw, pcre);
+    rc_c = rmatch(raw, preg);
     printf("\n");
   } else {
     line = in;
-    rc_c = rmatch(line, pcre);
+    rc_c = rmatch(line, preg);
     printf("\n");
   }
   if(rc_c < 0) {
