@@ -6,7 +6,7 @@
  * See http://mod-qos.sourceforge.net/ for further
  * details.
  *
- * Copyright (C) 2019 Pascal Buchbinder
+ * Copyright (C) 2023 Pascal Buchbinder
  *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
@@ -25,7 +25,7 @@
  *
  */
 
-static const char revision[] = "$Id: qsrotate.c 2542 2019-02-22 06:37:10Z pbuchbinder $";
+static const char revision[] = "$Id: qsrotate.c 2654 2022-05-13 09:12:42Z pbuchbinder $";
 
 #include <stdio.h>
 #include <string.h>
@@ -365,7 +365,7 @@ int main(int argc, char **argv) {
     cmd++;
   }
   m_cmd = calloc(1, strlen(cmd)+1);
-  strcpy(m_cmd, cmd); // copy as we wan't to pass it when forking
+  strcpy(m_cmd, cmd); // copy as we can't pass it when forking
 
   while(argc >= 1) {
     if(strcmp(*argv,"-o") == 0) {
