@@ -6,7 +6,7 @@
  * See http://mod-qos.sourceforge.net/ for further
  * details.
  *
- * Copyright (C) 2019 Pascal Buchbinder
+ * Copyright (C) 2023 Pascal Buchbinder
  *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
@@ -24,7 +24,7 @@
  * limitations under the License.
  */
 
-static const char revision[] = "$Id: qssign.c 2542 2019-02-22 06:37:10Z pbuchbinder $";
+static const char revision[] = "$Id: qssign.c 2654 2022-05-13 09:12:42Z pbuchbinder $";
 
 #include <stdio.h>
 #include <unistd.h>
@@ -37,9 +37,6 @@ static const char revision[] = "$Id: qssign.c 2542 2019-02-22 06:37:10Z pbuchbin
 #include <openssl/evp.h>
 #include <openssl/hmac.h>
 
-/* PCRE */
-#include <pcre.h>
-
 /* apr/apr-util */
 #define QS_USEAPR 1
 #include <apr.h>
@@ -50,6 +47,9 @@ static const char revision[] = "$Id: qssign.c 2542 2019-02-22 06:37:10Z pbuchbin
 #include <apr_file_io.h>
 #include <apr_time.h>
 
+#define PCRE2_CODE_UNIT_WIDTH 8
+#include <pcre2.h>
+
 #include "qs_util.h"
 #include "qs_apo.h"
 
@@ -65,7 +65,7 @@ static void (*m_end)(const char *, int) = NULL;
 static int m_end_pos = 0;
 static const char *m_sec = NULL;
 static const EVP_MD *m_evp;
-static const pcre *m_filter = NULL;
+static qs_regex_t *m_filter = NULL;
 
 typedef struct {
   const char* start_fmt;
@@ -310,7 +310,7 @@ void qs_signal_exit(int e) {
 }
 
 /**
- * Tries to find out a suiteable log line format which is used
+ * Tries to find out a suitable log line format which is used
  * to log sign end messages (so let the verifier known, that the
  * data ends nothing has been cut off).
  *
@@ -433,7 +433,7 @@ static void qs_sign(const char *sec) {
       qs_set_format(line);
       m_end(m_sec, 1);
     }
-    if(pcre_exec(m_filter, NULL, line, line_size, 0, 0, NULL, 0) >= 0) {
+    if(m_filter != NULL && qs_regexec_len(m_filter, line, line_len, 0, NULL, 0) >= 0) {
       printf("%s\n", line);
       fflush(stdout);
     } else {
@@ -517,7 +517,7 @@ static long qs_verify(const char *sec) {
       if(strcmp(m, sig) != 0) {
 	err++;
 	fprintf(stderr, "ERROR on line %ld: invalid signature\n", lineNumber);
-	/* message may be modified/currupt or inserted: next line may have
+	/* message may be modified/corrupt or inserted: next line may have
 	   the next sequence number (modified) or the same (inserted) */
 	nr_alt = m_nr + 1;
 	nr_alt_lineNumber = lineNumber + 1;
@@ -534,9 +534,9 @@ static long qs_verify(const char *sec) {
       } else {
 	if(m_nr != -1) {
 	  if(lineNumber == nr_alt_lineNumber) {
-	    // last line was modfied
+	    // last line was modified
 	    if(m_nr != msgSeqNr) {
-	      // and therefore, we also accept the next seqence number
+	      // and therefore, we also accept the next sequence number
 	      m_nr = nr_alt;
 	    }
 	    nr_alt = -1;
@@ -544,7 +544,7 @@ static long qs_verify(const char *sec) {
 	  }
 	  if(valid && isSpecialLine(line, QS_START)) {
 	    // new start line (graceful restart)
-	    // we expect now msg nummber 1
+	    // we expect now msg number 1
 	    // but still acept the old until we get the end marker
 	    nr_usr1_lineNumber = m_nr;
 	    m_nr = 1; 
@@ -566,7 +566,7 @@ static long qs_verify(const char *sec) {
 		fprintf(stderr, "ERROR on line %ld: wrong sequence (expect %."SEQDIG"ld)\n", lineNumber, m_nr);
 	      }
 	    } else {
-	      // well done - this is the sequence number we expet
+	      // well done - this is the sequence number we expect
 	    }
 	  }
 	} else if(m_logend) {
@@ -629,7 +629,7 @@ static void usage(char *cmd, int man) {
   }
   qs_man_print(man, "%s is a log data integrity check tool. It reads log data\n", cmd);
   qs_man_print(man, "from stdin (pipe) and writes the data to stdout adding a sequence\n");
-  qs_man_print(man, "number and signatur to ever log line.\n");
+  qs_man_print(man, "number and signature to ever log line.\n");
   printf("\n");
   if(man) {
     printf(".SH OPTIONS\n");
@@ -664,7 +664,7 @@ static void usage(char *cmd, int man) {
   if(man) printf("\n.TP\n");
   qs_man_print(man, "  -a 'sha1'|'sha256'\n");
   if(man) printf("\n");
-  qs_man_print(man, "     Specifes the algorithm to use. Default is sha1.\n");
+  qs_man_print(man, "     Specifies the algorithm to use. Default is sha1.\n");
   printf("\n");
   if(man) {
     printf(".SH EXAMPLE\n");
@@ -761,14 +761,12 @@ int main(int argc, const char * const argv[]) {
   }
 
   if(filter != NULL) {
-    const char *errptr = NULL;
-    int erroffset;
-    m_filter = pcre_compile(filter, 0, &errptr, &erroffset, NULL);
-    if(m_filter == NULL) {
-      fprintf(stderr, "failed to compile filter pattern <%s> at position %d,"
-            " reason: %s\n", filter, erroffset, errptr);
+    m_filter = apr_palloc(pool, sizeof(qs_regex_t));
+    if(qs_regcomp(m_filter, filter, 0) != 0) {
+      fprintf(stderr, "failed to compile filter pattern <%s>\n", filter);
       exit(1);
     }
+    apr_pool_pre_cleanup_register(pool, m_filter, qs_pregfree);
   }
 
   if(m_evp == NULL) {
